/* AsmBuilder_x8632nasm.hpp */

#ifndef ASMBUILDER_X8632NASM_H
#define ASMBUILDER_X8632NASM_H

#include "AsmBuilder.hpp"

class AsmBuilder_x8632nasm : public AsmBuilder {

public:

	/* Overridden methods */

	~AsmBuilder_x8632nasm() {}

	virtual void raw8(uint8_t val);

	virtual void raw16(uint16_t val);

	virtual void raw32(uint32_t val);

	virtual void rawData(const char* str, size_t len);

	virtual void rawData(std::string str);

	/* */

	virtual void pushReg(Register r);

	virtual void popReg(Register r);

	virtual void pushRegOffset(Register r, int32_t offset);

	virtual void movRegReg(Register dest, Register src);

	virtual void movRegImm32(Register dest, uint32_t val);

	virtual void movRegRegOffset(Register dest, Register srcptr, int32_t srcoffset);

	virtual void movRegOffsetReg(Register destptr, int32_t destoffset, Register src);

	virtual void addRegReg(Register accumulator, Register src);

	virtual void addRegImm32(Register accumulator, uint32_t val);

	virtual void clearRegister(Register r);

	/* */

	virtual void exportGlobalHere(std::string name);

	virtual void label(std::string name);

	virtual void jumpUnconditional(std::string label);

	virtual void jumpIfLesserOrEqual(std::string label);

	virtual void jumpIfLesser(std::string label);

	virtual void jumpIfEqual(std::string label);

	virtual void jumpIfNotEqual(std::string label);

	virtual void jumpIfGreater(std::string label);

	virtual void jumpIfGreaterOrEqual(std::string label);

	virtual void noop();

	/* */

	virtual void callByLabel(std::string label);

	virtual void callFromRegister(Register r);

	virtual void leave();

	virtual void returnFunc();

	virtual void returnFunc(uint16_t stacksize);

	/* */

	virtual bool renderObject(std::string& ret);

};

#endif
