/* main.cpp */

#include "AsmReader.hpp"
#include "AsmBuilder_x8632nasm.hpp"
#include "AsmBuilder_x8632raw.hpp"
#include "AsmBuilder_x8632obj.hpp"

#include <iostream>
#include <fstream>
#include <memory>

void usage() {
	std::cout <<
		"Usage: \n"
		"  asmbuilder [options] \n"
		"\n"
		"Options: \n"
		"  --help, --usage   Display this message\n"
		"  -i {filename}     Set input file (default stdin)\n"
		"  -o {filename}     Set output file (default stdout)\n"
		"  --x8632nasm       Use the x8632nasm output engine\n"
		"  --x8632raw        Use the x8632raw output engine\n"
		"  --x8632obj        Use the x8632obj output engine\n"
	;
}
struct noop {
	void operator()(...) const {}
};
int main(int argc, char** argv) {

	std::shared_ptr<AsmBuilder> builder;
	std::shared_ptr<std::istream> input;
	std::shared_ptr<std::ostream> output;

	for (int i = 1; i < argc; ++i) {

		if (!strcmp(argv[i], "--x8632nasm")) {
			builder.reset(new AsmBuilder_x8632nasm());

		} else if (!strcmp(argv[i], "--x8632raw")) {
			builder.reset(new AsmBuilder_x8632raw());

		} else if (!strcmp(argv[i], "--x8632obj")) {
			builder.reset(new AsmBuilder_x8632obj());

		} else if (!strcmp(argv[i], "-i")) {
			++i;
			input.reset(new std::ifstream(argv[i]));
			if (! input->good()) {
				std::cerr << "FATAL: Error reading file '" << argv[i] << "'\n";
				return 1;
			}

		} else if (!strcmp(argv[i], "-o")) {
			++i;
			output.reset(new std::ofstream(argv[i]));
			if (! output->good()) {
				std::cerr << "FATAL: Error writing to file '" << argv[i] << "'\n";
				return 1;
			}

		} else if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "--usage")) {
			usage();
			return 0;

		} else {
			std::cerr << "FATAL: Unrecognised argument '" << argv[i] << "'\n";
			return 1;

		}

	}

	if (builder == nullptr) {
		std::cerr << "FATAL: No output engine selected\n";
		return 1;
	}

	if (input == nullptr) {
		input.reset(&std::cin, noop());
	}

	if (output == nullptr) {
		output.reset(&std::cout, noop());
	}

	builder->changeSection(".text"); // default to .text section

	AsmReader asmr;
	if (asmr.process(input.get(), output.get(), builder.get())) {
		return 0;

	} else {
		std::cerr << builder->errors.str();
		std::cerr << asmr.errors.str();
		return 1;

	}
}
